package com.YuyeTech.HeartRate.game.levelInfo.service;

import com.YuyeTech.HeartRate.common.service.CommonService;
import com.YuyeTech.HeartRate.game.levelInfo.projo.LevelCondition;
import com.YuyeTech.HeartRate.game.levelInfo.projo.LevelHonor;
import com.YuyeTech.HeartRate.game.levelInfo.projo.LevelJump;
import com.YuyeTech.HeartRate.dto.LevelJumpDTO;
import com.YuyeTech.HeartRate.game.levelInfo.vo.LevelJumpVo;
import org.springframework.data.domain.Sort;

import java.util.List;
import java.util.Map;

/**
 * @author:slambb
 * @date:2021/8/4
 */
public interface LevelJumpService extends CommonService<LevelJumpVo, LevelJump, Integer> {

    List<LevelJump> findLevelJumpsByNumberTypeAndType(Integer numberType, Integer type, Sort.Direction direction);

    List<LevelJumpDTO> findLevelJumpDTOsByNumberType(Integer numberType, Sort.Direction direction);

    Map findLevelJumpsAll(String userId);

    Map findLevels(String userId);

    List<LevelJump> findLevelJumpsAll();

    LevelJump findById(Integer id);

    //查询全部条件，包括解锁和通关
    List<LevelCondition> findAllLevelCondition();

    //查询全部荣誉
    List<LevelHonor> findAllLevelHonor();
}
